package edu.rice.cs.mint.runtime;

public class Utilities {
    /**
     * Translate the return value from Class.getName() to a valid Java type string.
     * For normal class names, this returns the argument, with '$' replaced by '.'.
     * For arrays, i.e. strings starting with '[', it translates the string according to the rules in Class.getName().
     * @param typeString return value from Class.getName()
     * @return Java type string
     */
    public static String getTypeString(String typeString) {
        if (!typeString.startsWith("[")) return typeString.replace('$','.');
        
        int isArray = 0;
        int ndx = 0;
        StringBuilder x = new StringBuilder();

        while('[' == typeString.charAt(ndx)) {
            ++isArray;
            ++ndx;
        }

        switch(typeString.charAt(ndx)) {
            case 'B':
                x.append("byte");
                break;
            case 'C':
                x.append("char");
                break;
            case 'D':
                x.append("double");
                break;
            case 'F':
                x.append("float");
                break;
            case 'I':
                x.append("int");
                break;
            case 'J':
                x.append("long");
                break;
            case 'L':
                for(int i = ndx + 1; i < typeString.indexOf((int)';'); ++i) {
                    if (('/' == typeString.charAt(i)) ||
                        ('$' == typeString.charAt(i))) {
                        x.append('.');
                    }
                    else {
                        x.append(typeString.charAt(i));
                    }
                }
                break;
            case 'V':
                x.append("void");
                break;
            case 'S':
                x.append("short");
                break;
            case 'Z':
                x.append("boolean");
                break;
        }
        while(0 < isArray) {
            x.append("[]");
            isArray--;
        }
        return x.toString();
    }
}
